@@ -15,13 +15,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 #include <stdio.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "connection.h"
@@ -30,6 +34,8 @@
 #include "arrows.h"
 #include "diamenu.h"
 #include "text.h"
+#include "connection.h"
+#include "properties.h"
 
 #include "pixmaps/flow.xpm"
 
@@ -75,7 +81,6 @@ struct _FlowDialog {
 
 static Font *flow_font = NULL;
 
-static FlowDialog *properties_dialog;
 static FlowDialog *defaults_dialog;
 
 /* Remember the most recently applied flow type and use it to
@@ -98,14 +103,17 @@ static real flow_distance_from(Flow *flow, Point *point);
 static void flow_update_data(Flow *flow);
 static void flow_destroy(Flow *flow);
 static Object *flow_copy(Flow *flow);
-static GtkWidget *flow_get_properties(Flow *flow);
-static ObjectChange *flow_apply_properties(Flow *flow);
 static GtkWidget *flow_get_defaults(void);
 static void flow_apply_defaults(void);
 static void flow_save(Flow *flow, ObjectNode obj_node,
 		      const char *filename);
 static Object *flow_load(ObjectNode obj_node, int version,
 			 const char *filename);
+static PropDescription *flow_describe_props(Flow *mes);
+static void
+flow_get_props(Flow * flow, Property *props, guint nprops);
+static void
+flow_set_props(Flow * flow, Property *props, guint nprops);
 static DiaMenu *flow_get_object_menu(Flow *flow, Point *clickedpoint) ;
 
 
@@ -134,11 +142,59 @@ static ObjectOps flow_ops = {
   (CopyFunc)            flow_copy,
   (MoveFunc)            flow_move,
   (MoveHandleFunc)      flow_move_handle,
-  (GetPropertiesFunc)   flow_get_properties,
-  (ApplyPropertiesFunc) flow_apply_properties,
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
   (ObjectMenuFunc)      flow_get_object_menu,
+  (DescribePropsFunc)   flow_describe_props,
+  (GetPropsFunc)        flow_get_props,
+  (SetPropsFunc)        flow_set_props,
 };
 
+static PropEnumData prop_flow_type_data[] = {
+  { N_("Energy"),FLOW_ENERGY },
+  { N_("Material"),FLOW_MATERIAL },
+  { N_("Signal"),FLOW_SIGNAL },
+  { NULL, 0 } 
+};
+
+static PropDescription flow_props[] = {
+  OBJECT_COMMON_PROPERTIES,
+  { "type", PROP_TYPE_ENUM, PROP_FLAG_VISIBLE,
+    N_("Type:"), NULL, prop_flow_type_data },
+  PROP_DESC_END
+};
+
+static PropDescription *
+flow_describe_props(Flow *mes)
+{
+  if (flow_props[0].quark == 0)
+    prop_desc_list_calculate_quarks(flow_props);
+  return flow_props;
+
+}
+
+static PropOffset flow_offsets[] = {
+  OBJECT_COMMON_PROPERTIES_OFFSETS,
+  { "type", PROP_TYPE_ENUM, offsetof(Flow, type) },
+  { NULL, 0, 0 }
+};
+
+static void
+flow_get_props(Flow * flow, Property *props, guint nprops)
+{
+  object_get_props_from_offsets(&flow->connection.object, 
+                                flow_offsets, props, nprops);
+}
+
+static void
+flow_set_props(Flow *flow, Property *props, guint nprops)
+{
+  object_set_props_from_offsets(&flow->connection.object, 
+                                flow_offsets, props, nprops);
+  flow_update_data(flow);
+}
+
+
 static real
 flow_distance_from(Flow *flow, Point *point)
 {
@@ -161,7 +217,7 @@ flow_select(Flow *flow, Point *clicked_point,
 	    Renderer *interactive_renderer)
 {
   text_set_cursor(flow->text, clicked_point, interactive_renderer);
-  text_grab_focus(flow->text, (Object *)flow);
+  text_grab_focus(flow->text, &flow->connection.object);
 
   connection_update_handles(&flow->connection);
 }
@@ -313,18 +369,20 @@ flow_create(Point *startpoint,
   Flow *flow;
   Connection *conn;
   Object *obj;
+  ConnectionBBExtras *extra;
   Point p ;
   Point n ;
 
-  flow = g_malloc(sizeof(Flow));
+  flow = g_malloc0(sizeof(Flow));
 
   conn = &flow->connection;
   conn->endpoints[0] = *startpoint;
   conn->endpoints[1] = *startpoint;
   conn->endpoints[1].x += 1.5;
  
-  obj = (Object *) flow;
-  
+  obj = &conn->object;
+  extra = &conn->extra_spacing;
+
   obj->type = &flow_type;
   obj->ops = &flow_ops;
   
@@ -377,12 +435,17 @@ flow_create(Point *startpoint,
   flow->text_handle.connect_type = HANDLE_NONCONNECTABLE;
   flow->text_handle.connected_to = NULL;
   obj->handles[2] = &flow->text_handle;
+
+  extra->start_long = 
+    extra->end_long =
+    extra->start_trans = FLOW_WIDTH/2.0;
+  extra->end_trans = MAX(FLOW_WIDTH, FLOW_ARROWLEN) / 2.0;
   
   flow_update_data(flow);
 
   *handle1 = obj->handles[0];
   *handle2 = obj->handles[1];
-  return (Object *)flow;
+  return &flow->connection.object;
 }
 
 
@@ -402,9 +465,9 @@ flow_copy(Flow *flow)
   
   conn = &flow->connection;
   
-  newflow = g_malloc(sizeof(Flow));
+  newflow = g_malloc0(sizeof(Flow));
   newconn = &newflow->connection;
-  newobj = (Object *) newflow;
+  newobj = &newconn->object;
 
   connection_copy(conn, newconn);
 
@@ -414,7 +477,7 @@ flow_copy(Flow *flow)
   newflow->text = text_copy(flow->text);
   newflow->type = flow->type;
 
-  return (Object *)newflow;
+  return &newflow->connection.object;
 }
 
 
@@ -422,7 +485,7 @@ static void
 flow_update_data(Flow *flow)
 {
   Connection *conn = &flow->connection;
-  Object *obj = (Object *) flow;
+  Object *obj = &conn->object;
   Rectangle rect;
   Color* color = NULL;
   
@@ -451,12 +514,6 @@ flow_update_data(Flow *flow)
   /* Add boundingbox for text: */
   text_calc_boundingbox(flow->text, &rect) ;
   rectangle_union(&obj->bounding_box, &rect);
-
-  /* fix boundingbox for flow_width and arrow: */
-  obj->bounding_box.top -= FLOW_WIDTH/2 + FLOW_ARROWLEN;
-  obj->bounding_box.left -= FLOW_WIDTH/2 + FLOW_ARROWLEN;
-  obj->bounding_box.bottom += FLOW_WIDTH/2 + FLOW_ARROWLEN;
-  obj->bounding_box.right += FLOW_WIDTH/2 + FLOW_ARROWLEN;
 }
 
 
@@ -478,14 +535,16 @@ flow_load(ObjectNode obj_node, int version, const char *filename)
   AttributeNode attr;
   Connection *conn;
   Object *obj;
+  ConnectionBBExtras *extra;
 
   if (flow_font == NULL)
     flow_font = font_getfont("Helvetica-Oblique");
 
-  flow = g_malloc(sizeof(Flow));
+  flow = g_malloc0(sizeof(Flow));
 
   conn = &flow->connection;
-  obj = (Object *) flow;
+  obj = &conn->object;
+  extra = &conn->extra_spacing;
 
   obj->type = &flow_type;
   obj->ops = &flow_ops;
@@ -508,68 +567,17 @@ flow_load(ObjectNode obj_node, int version, const char *filename)
   flow->text_handle.connect_type = HANDLE_NONCONNECTABLE;
   flow->text_handle.connected_to = NULL;
   obj->handles[2] = &flow->text_handle;
+
+  extra->start_long = 
+    extra->end_long =
+    extra->start_trans = FLOW_WIDTH/2.0;
+  extra->end_trans = MAX(FLOW_WIDTH, FLOW_ARROWLEN) / 2.0;
   
   flow_update_data(flow);
   
-  return (Object *)flow;
+  return &flow->connection.object;
 }
 
-
-static ObjectChange *
-flow_apply_properties(Flow *flow)
-{
-  FlowDialog *prop_dialog;
-  
-  prop_dialog = properties_dialog;
-
-  text_set_string(flow->text,
-                  gtk_editable_get_chars( GTK_EDITABLE(prop_dialog->text),
-					  0, -1));
-
-  if (GTK_TOGGLE_BUTTON(prop_dialog->m_energy)->active)
-    flow->type = FLOW_ENERGY;
-  else if (GTK_TOGGLE_BUTTON( prop_dialog->m_material )->active) 
-    flow->type = FLOW_MATERIAL;
-  else if (GTK_TOGGLE_BUTTON( prop_dialog->m_signal )->active) 
-    flow->type = FLOW_SIGNAL;
-
-  flow_update_data(flow);
-
-  return NULL;
-}
-
-static void
-fill_in_dialog(Flow *flow)
-{
-  FlowDialog *prop_dialog;
-  GtkToggleButton *button=NULL;
-
-  prop_dialog = properties_dialog;
-
-  gtk_text_set_point( GTK_TEXT(prop_dialog->text), 0 ) ;
-  gtk_text_forward_delete( GTK_TEXT(prop_dialog->text), 
-			   gtk_text_get_length(GTK_TEXT(prop_dialog->text))) ;
-  gtk_text_insert( GTK_TEXT(prop_dialog->text),
-                   NULL, NULL, NULL,
-                   text_get_string_copy(flow->text),
-                   -1);
-
-  switch (flow->type) {
-  case FLOW_ENERGY:
-    button = GTK_TOGGLE_BUTTON(prop_dialog->m_energy);
-    break;
-  case FLOW_MATERIAL:
-    button = GTK_TOGGLE_BUTTON(prop_dialog->m_material);
-    break;
-  case FLOW_SIGNAL:
-    button = GTK_TOGGLE_BUTTON(prop_dialog->m_signal);
-    break;
-  }
-  if (button)
-    gtk_toggle_button_set_active(button, TRUE);
-}
-
-
 static void
 fill_in_defaults_dialog(void)
 {
@@ -603,77 +611,6 @@ fill_in_defaults_dialog(void)
     gtk_toggle_button_set_active(button, TRUE);
 }
 
-
-static GtkWidget *
-flow_get_properties(Flow *flow)
-{
-  FlowDialog *prop_dialog;
-  GtkWidget *dialog;
-  GtkWidget *entry;
-  GtkWidget *hbox;
-  GtkWidget *label;
-  GSList *group;
-
-  if (properties_dialog == NULL) {
-
-    prop_dialog = g_new(FlowDialog, 1);
-    properties_dialog = prop_dialog;
-    
-    dialog = gtk_vbox_new(FALSE, 0);
-    gtk_object_ref(GTK_OBJECT(dialog));
-    gtk_object_sink(GTK_OBJECT(dialog));
-    prop_dialog->dialog = dialog;
-    
-    hbox = gtk_hbox_new(FALSE, 5);
-
-    label = gtk_label_new(_("Flow:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    entry = gtk_text_new(NULL, NULL);
-    prop_dialog->text = entry ;
-    gtk_text_set_editable(GTK_TEXT(entry), TRUE);
-    gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
-    gtk_widget_show (label);
-    gtk_widget_show (entry);
-    gtk_box_pack_start (GTK_BOX (dialog), hbox, TRUE, TRUE, 0);
-    gtk_widget_show(hbox);
-
-    label = gtk_hseparator_new ();
-    gtk_box_pack_start (GTK_BOX (dialog), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-
-    label = gtk_label_new(_("Flow type:"));
-    gtk_box_pack_start (GTK_BOX (dialog), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-
-    /* */
-    prop_dialog->m_energy = gtk_radio_button_new_with_label (NULL, _("Energy"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_energy, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_energy);
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prop_dialog->m_energy), TRUE);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_energy));
-
-    prop_dialog->m_material = gtk_radio_button_new_with_label(group, _("Material"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_material, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_material);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_material));
-
-    prop_dialog->m_signal = gtk_radio_button_new_with_label(group, _("Signal"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_signal, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_signal);
-
-#if 0
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_signal));
-#endif
-  }
-  
-  fill_in_dialog(flow);
-  gtk_widget_show (properties_dialog->dialog);
-
-  return properties_dialog->dialog;
-}
-
 static void
 flow_update_defaults( Flow* flow, char update_text )
 {
