@@ -15,11 +15,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "orth_conn.h"
@@ -146,14 +150,14 @@ static PropOffset zigzagline_offsets[] = {
 static void
 zigzagline_get_props(Zigzagline *zigzagline, Property *props, guint nprops)
 {
-  object_get_props_from_offsets((Object *)zigzagline, zigzagline_offsets,
+  object_get_props_from_offsets(&zigzagline->orth.object, zigzagline_offsets,
 				props, nprops);
 }
 
 static void
 zigzagline_set_props(Zigzagline *zigzagline, Property *props, guint nprops)
 {
-  object_set_props_from_offsets((Object *)zigzagline, zigzagline_offsets,
+  object_set_props_from_offsets(&zigzagline->orth.object, zigzagline_offsets,
 				props, nprops);
   zigzagline_update_data(zigzagline);
 }
@@ -237,9 +241,9 @@ zigzagline_create(Point *startpoint,
   Object *obj;
 
   /*zigzagline_init_defaults();*/
-  zigzagline = g_malloc(sizeof(Zigzagline));
+  zigzagline = g_malloc0(sizeof(Zigzagline));
   orth = &zigzagline->orth;
-  obj = (Object *) zigzagline;
+  obj = &orth->object;
   
   obj->type = &zigzagline_type;
   obj->ops = &zigzagline_ops;
@@ -257,7 +261,7 @@ zigzagline_create(Point *startpoint,
   
   *handle1 = orth->handles[0];
   *handle2 = orth->handles[orth->numpoints-2];
-  return (Object *)zigzagline;
+  return &zigzagline->orth.object;
 }
 
 static void
@@ -275,50 +279,42 @@ zigzagline_copy(Zigzagline *zigzagline)
   
   orth = &zigzagline->orth;
  
-  newzigzagline = g_malloc(sizeof(Zigzagline));
+  newzigzagline = g_malloc0(sizeof(Zigzagline));
   neworth = &newzigzagline->orth;
-  newobj = (Object *) newzigzagline;
+  newobj = &neworth->object;
 
   orthconn_copy(orth, neworth);
 
   newzigzagline->line_color = zigzagline->line_color;
   newzigzagline->line_width = zigzagline->line_width;
   newzigzagline->line_style = zigzagline->line_style;
+  newzigzagline->dashlength = zigzagline->dashlength;
   newzigzagline->start_arrow = zigzagline->start_arrow;
   newzigzagline->end_arrow = zigzagline->end_arrow;
 
-  return (Object *)newzigzagline;
+  return &newzigzagline->orth.object;
 }
 
 static void
 zigzagline_update_data(Zigzagline *zigzagline)
 {
   OrthConn *orth = &zigzagline->orth;
-  Object *obj = (Object *) zigzagline;
+  OrthConnBBExtras *extra = &orth->extra_spacing;
 
   orthconn_update_data(&zigzagline->orth);
     
+  extra->start_long = 
+    extra->end_long = 
+    extra->middle_trans = zigzagline->line_width/2.0;
+  extra->start_trans = (zigzagline->line_width / 2.0);
+  extra->end_trans = (zigzagline->line_width / 2.0);
+
+  if (zigzagline->start_arrow.type != ARROW_NONE) 
+    extra->start_trans = MAX(extra->start_trans,zigzagline->start_arrow.width);
+  if (zigzagline->end_arrow.type != ARROW_NONE) 
+    extra->end_trans = MAX(extra->end_trans,zigzagline->end_arrow.width);
+  
   orthconn_update_boundingbox(orth);
-  /* fix boundingbox for line_width: */
-  obj->bounding_box.top -= zigzagline->line_width/2;
-  obj->bounding_box.left -= zigzagline->line_width/2;
-  obj->bounding_box.bottom += zigzagline->line_width/2;
-  obj->bounding_box.right += zigzagline->line_width/2;
-
-  /* Fix boundingbox for arrowheads */
-  if (zigzagline->start_arrow.type != ARROW_NONE ||
-      zigzagline->end_arrow.type != ARROW_NONE) {
-    real arrow_width = 0.0;
-    if (zigzagline->start_arrow.type != ARROW_NONE)
-      arrow_width = zigzagline->start_arrow.width;
-    if (zigzagline->end_arrow.type != ARROW_NONE)
-      arrow_width = MAX(arrow_width, zigzagline->start_arrow.width);
-
-    obj->bounding_box.top -= arrow_width;
-    obj->bounding_box.left -= arrow_width;
-    obj->bounding_box.bottom += arrow_width;
-    obj->bounding_box.right += arrow_width;
-  }
 }
 
 static ObjectChange *
@@ -414,10 +410,10 @@ zigzagline_load(ObjectNode obj_node, int version, const char *filename)
   Object *obj;
   AttributeNode attr;
 
-  zigzagline = g_malloc(sizeof(Zigzagline));
+  zigzagline = g_malloc0(sizeof(Zigzagline));
 
   orth = &zigzagline->orth;
-  obj = (Object *) zigzagline;
+  obj = &orth->object;
   
   obj->type = &zigzagline_type;
   obj->ops = &zigzagline_ops;
@@ -472,5 +468,5 @@ zigzagline_load(ObjectNode obj_node, int version, const char *filename)
   
   zigzagline_update_data(zigzagline);
 
-  return (Object *)zigzagline;
+  return &zigzagline->orth.object;
 }
