@@ -20,25 +20,31 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
 #include <glib.h>
 
-#include "config.h"
 #include "intl.h"
 #include "message.h"
 #include "geometry.h"
 #include "render.h"
 #include "filter.h"
 
+#ifdef THIS_IS_PROBABLY_DEAD_CODE
 static const gchar *dia_version_string = "Dia-" VERSION;
+#endif
+
 #define IS_ODD(n) (n & 0x01)
 
 static void
 init_fonts(void)
 {
-
+    
 }
 
 /* --- dxf line attributes --- */
@@ -499,7 +505,8 @@ draw_image(Rendererdxf *renderer,
 }
 
 static void
-export_dxf(DiagramData *data, const gchar *filename, const gchar *diafilename)
+export_dxf(DiagramData *data, const gchar *filename, 
+           const gchar *diafilename, void* user_data)
 {
     Rendererdxf *renderer;
     FILE *file;
